module Moveis3D
    module Tools
        class CriarComponenteTool
            def activate
                @model = Sketchup.active_model
                @selection = @model.selection
                @view = @model.active_view

                cursor_path = File.join(__dir__, "18mm.png")

                if File.exist?(cursor_path)
                    @cursor_id = UI.create_cursor(cursor_path, 16, 16)
                else
                    UI.messagebox("Arquivo de cursor não encontrado: #{cursor_path}")
                    @cursor_id = 635 # Cursor padrão como fallback
                end

                UI.set_cursor(@cursor_id) # Define o cursor ao ativar a ferramenta
            end

            def onMouseMove(flags, x, y, view)
                UI.set_cursor(@cursor_id) if @cursor_id
            end

            def onSetCursor
                UI.set_cursor(@cursor_id) if @cursor_id
                true # Retorna `true` para indicar que o cursor foi definido corretamente
            end

            def deactivate(view)
                view.lock_inference
            end

            def onLButtonDown(flags, x, y, view)
                # Obtém o ponto clicado na tela
                ph = view.pick_helper
                ph.do_pick(x, y)

                # Verifica se o clique foi em uma face
                face = ph.best_picked
                if face.is_a?(Sketchup::Face)
                    criar_componente(@model, face)
                else
                    UI.messagebox("Por favor, clique em uma face retangular para criar o componente.")
                end
            end

            private

            def criar_componente(model, face)
                model.start_operation("Criar Componente 18mm", true)

                begin
                    return unless face.is_a?(Sketchup::Face) && face.valid?

                    # Criar um grupo temporário para armazenar a nova geometria
                    grupo_temporario = model.active_entities.add_group
                    entidades_temporarias = grupo_temporario.entities

                    # Criar uma cópia da face principal no grupo
                    nova_face = entidades_temporarias.add_face(face.outer_loop.vertices)

                    # Verificar se a face tem buracos internos e removê-los
                    face.loops.each do |loop|
                        next if loop.outer? # Mantém apenas o contorno externo

                        # Verifica se o loop ainda está válido antes de adicionar
                        if nova_face.valid? && loop.edges.all?(&:valid?)
                            nova_face.add_inner_loop(loop.edges.map(&:start))
                        else
                            puts "⚠️ Loop interno ignorado por conter entidades inválidas."
                        end
                    end

                    # Excluir as faces internas (buracos) logo no início
                    face.loops.each do |loop|
                        next if loop.outer?  # Ignora o contorno externo (não remove)

                        # Excluir apenas a face interna (retângulo menor), não a face do retângulo principal
                        loop.edges.each do |edge|
                            edge.faces.each do |inner_face|
                                if inner_face != face && inner_face.valid? # Garante que não seja a face principal
                                    model.active_entities.erase_entities(inner_face)  # Excluir a face interna
                                end
                            end
                        end
                    end

                    # Remover todas as entidades conectadas do modelo original
                    model.active_entities.erase_entities(obter_entidades_conectadas(face))

                    # Aplicar push/pull para criar espessura de 18mm
                    nova_face.pushpull(-18.mm)

                    # Transformar o grupo em um componente
                    instancia = grupo_temporario.to_component
                    instancia.definition.name = "Componente 18mm"

                    # Aplicar materiais
                    aplicar_material(instancia, "Branco TX", "MDF Cru")

                    # Selecionar o novo componente
                    model.selection.clear
                    model.selection.add(instancia)

                    puts "✅ Componente criado com sucesso!"

                    model.commit_operation

                    # Abrir o HtmlDialog da ferramenta EditComponentDialog
                    Moveis3D::EditComponentDialog.show_dialog

                    Sketchup.active_model.select_tool(Moveis3D::RotateFurnitureTool.new)

                rescue NoMethodError => e
                    model.abort_operation
                    UI.messagebox("Erro ao criar componente: Os retângulos internos precisam estar conectados com alguma aresta do retângulo principal.")
                    puts "Erro ao criar componente: #{e.message}" # Mantém o log do erro para referência futura
                rescue => e
                    model.abort_operation
                    UI.messagebox("Erro ao criar componente: #{e.message}")
                end
            end

            # Método para obter todas as entidades conectadas à face original (recursivo)
            def obter_entidades_conectadas(face)
                return [] unless face.is_a?(Sketchup::Face) && face.valid?

                entidades_conectadas = []
                entidades_processadas = {}

                # Rastreia todas as entidades conectadas, incluindo buracos internos
                queue = [face]

                while queue.any?
                    entidade = queue.pop
                    next if entidades_processadas[entidade] || !entidade.valid?

                    entidades_conectadas << entidade
                    entidades_processadas[entidade] = true

                    case entidade
                    when Sketchup::Face
                        # Adiciona todas as arestas da face e outras faces conectadas a essas arestas
                        entidade.edges.each do |edge|
                            queue << edge
                            edge.faces.each { |f| queue << f unless f == face }
                        end
                    when Sketchup::Edge
                        # Adiciona todas as faces conectadas à aresta
                        entidade.faces.each { |f| queue << f }
                    end
                end

                entidades_conectadas
            end

            def aplicar_material(componente, nome_material_frente, nome_material_lateral)
                materiais = componente.model.materials

                # Aplica o material branco nas faces maiores
                material_frente = materiais[nome_material_frente] || materiais.add(nome_material_frente)
                material_frente.color = "#f3f3f3" # Define um tom de branco

                # Verifica se o material marrom já existe
                material_lateral = materiais[nome_material_lateral]

                if material_lateral.nil?
                    # Se o material marrom não existir, cria um novo
                    material_lateral = materiais.add(nome_material_lateral)
                    material_lateral.color = "#8B4513" # Define um tom de marrom (MDF cru)
                end

                # Obtém todas as faces do componente
                faces = componente.definition.entities.grep(Sketchup::Face)

                # Ordena as faces por área (da maior para a menor)
                faces_ordenadas = faces.sort_by { |face| -face.area }

                # Aplica o material branco nas duas faces maiores
                faces_ordenadas[0..1].each { |face| face.material = material_frente }

                # Aplica o material marrom nas outras faces (bordas de 18mm)
                faces_ordenadas[2..-1].each { |face| face.material = material_lateral }
            end
        end
    end
end
