class CuttingPlanPDFGenerator {
    constructor(cuttingPlan) {
        this.cuttingPlan = cuttingPlan;
        this.pdf = new jspdf.jsPDF({
            orientation: 'landscape',
            unit: 'mm',
            format: 'a4'
        });
        this.pageWidth = this.pdf.internal.pageSize.width;
        this.pageHeight = this.pdf.internal.pageSize.height;
        this.margin = 10;
        this.currentY = this.margin + 10;

        const maxSheetWidth = this.pageWidth - (this.margin * 2);
        this.scale = maxSheetWidth / this.cuttingPlan.sheetWidth;

        this.borderColors = [
            [0, 123, 255],    // Azul
            [220, 53, 69],    // Vermelho
            [40, 167, 69],    // Verde
            [255, 193, 7],    // Amarelo
            [111, 66, 193],   // Roxo
            [23, 162, 184],   // Ciano
            [102, 16, 242],   // Índigo
            [232, 62, 140],   // Rosa
            [253, 126, 20],   // Laranja
            [32, 201, 151]    // Turquesa
        ];
    }

    generatePDF() {
        try {
            const modelName = document.querySelector('header h2').textContent.split('-')[1]?.trim() || 'Projeto';

            // Processamento de todas as chapas
            Object.entries(this.cuttingPlan.sheets).forEach(([material, sheets], materialIndex) => {
                sheets.forEach((sheet, sheetIndex) => {
                    // Adiciona nova página apenas se não for a primeira chapa
                    if (materialIndex > 0 || sheetIndex > 0) {
                        this.pdf.addPage();
                    }

                    // Reset posição Y e adiciona cabeçalho
                    this.currentY = this.margin + 10;
                    this.pdf.setFontSize(12);
                    this.pdf.text(`${modelName} - ${new Date('2025-03-16').toLocaleDateString()}`,
                                  this.pageWidth / 2, this.margin, { align: 'center' });

                    // Desenha a chapa
                    this.drawSheet(sheet, sheetIndex, material, sheets.length);
                });
            });

            // Salva o PDF
            this.pdf.save(`Plano de Corte - ${modelName}.pdf`);

        } catch (error) {
            console.error('Erro ao gerar PDF:', error);
            alert('Erro ao gerar o PDF. Verifique o console para mais detalhes.');
        }
    }

    drawSheet(sheet, sheetIndex, material, totalSheets) {
        const startX = this.margin;
        const sheetWidthScaled = this.cuttingPlan.sheetWidth * this.scale;
        const sheetHeightScaled = this.cuttingPlan.sheetHeight * this.scale;

        // Título da chapa
        this.pdf.setFontSize(12);
        this.pdf.text(`Chapa ${material} - ${sheetIndex + 1}/${totalSheets}`,
                      startX, this.currentY - 5);

        // Contorno da chapa
        this.pdf.setDrawColor(0);
        this.pdf.rect(startX, this.currentY, sheetWidthScaled, sheetHeightScaled);

        // Dimensões totais da chapa (fora da chapa)
        this.pdf.setTextColor(0);
        this.pdf.setFontSize(8);

        // Largura total da chapa (em cima)
        this.pdf.text(`${Math.round(this.cuttingPlan.sheetWidth)}`,
                      startX + (sheetWidthScaled / 2),
                      this.currentY - 1,
                      { align: 'center' });

        // Altura total da chapa (lado esquerdo)
        this.pdf.text(`${Math.round(this.cuttingPlan.sheetHeight)}`,
                      startX - 1,
                      this.currentY + (sheetHeightScaled / 2),
                      { align: 'center', angle: 90 });

        // Encontrar as maiores peças por linha e coluna
        const rowGroups = new Map(); // Agrupar por coordenada Y
        const colGroups = new Map(); // Agrupar por coordenada X

        sheet.placements.forEach(piece => {
            // Arredondar as coordenadas para agrupar peças próximas
            const roundedY = Math.round(piece.y);
            const roundedX = Math.round(piece.x);

            // Agrupar por linhas (Y)
            if (!rowGroups.has(roundedY)) {
                rowGroups.set(roundedY, []);
            }
            rowGroups.get(roundedY).push(piece);

            // Agrupar por colunas (X)
            if (!colGroups.has(roundedX)) {
                colGroups.set(roundedX, []);
            }
            colGroups.get(roundedX).push(piece);
        });

        // Desenhar dimensões dos primeiros cortes
        this.pdf.setTextColor(0, 0, 255); // Azul para os primeiros cortes
        this.pdf.setFontSize(7);

        // Marcar alturas das maiores peças por linha
        Array.from(rowGroups.values()).forEach(piecesInRow => {
            // Encontrar a peça mais alta da linha
            const tallestPiece = piecesInRow.reduce((prev, current) =>
            prev.height > current.height ? prev : current
            );

            const xPos = startX - 1; // 1mm à esquerda da chapa
            const yPos = this.currentY + (tallestPiece.y * this.scale) + (tallestPiece.height * this.scale / 2);

            this.pdf.text(`${Math.round(tallestPiece.height)}`, xPos, yPos, {
                align: 'right',
                angle: 90
            });
        });

        // Marcar larguras das maiores peças por coluna
        Array.from(colGroups.values()).forEach(piecesInCol => {
            // Encontrar a peça mais larga da coluna
            const widestPiece = piecesInCol.reduce((prev, current) =>
            prev.width > current.width ? prev : current
            );

            const xPos = startX + (widestPiece.x * this.scale) + (widestPiece.width * this.scale / 2);
            const yPos = this.currentY - 1; // 1mm acima da chapa

            this.pdf.text(`${Math.round(widestPiece.width)}`, xPos, yPos, {
                align: 'center'
            });
        });




        // Desenha os espaços disponíveis (sobras)
        if (sheet.spaces) {
            sheet.spaces.forEach(space => {
                if (space.width > this.cuttingPlan.sawKerf * 2 &&
                    space.height > this.cuttingPlan.sawKerf * 2) {

                    const x = startX + (space.x * this.scale);
                const y = this.currentY + (space.y * this.scale);
                const width = space.width * this.scale;
                const height = space.height * this.scale;

                this.pdf.setDrawColor(150, 150, 150);
                this.pdf.rect(x, y, width, height);

                this.pdf.setFontSize(6);
                this.pdf.setTextColor(100);
                const dimensionText = `${Math.round(space.width)}×${Math.round(space.height)}`;
                this.pdf.text(dimensionText, x + (width / 2), y + (height / 2), { align: 'center' });
                    }
            });
        }

        // Desenha as peças
        sheet.placements.forEach((piece, pieceIndex) => {
            const x = startX + (piece.x * this.scale);
            const y = this.currentY + (piece.y * this.scale);
            const width = piece.width * this.scale;
            const height = piece.height * this.scale;

            // Desenha a peça
            const borderColor = this.borderColors[pieceIndex % this.borderColors.length];
            this.pdf.setDrawColor(...borderColor);
            this.pdf.setLineWidth(0.3);
            this.pdf.rect(x, y, width, height);

            // Nome da peça - sempre no sentido da maior dimensão
            this.pdf.setTextColor(0);
            this.pdf.setFontSize(8);

            if (height > width) {
                // Texto rotacionado 90 graus
                this.pdf.text(piece.name, x + (width / 2), y + (height / 2), {
                 //   align: 'center',
                    angle: 90
                });
            } else {
                // Texto horizontal
                this.pdf.text(piece.name, x + (width / 2), y + (height / 2), {
               //     align: 'center'
                });
            }



            // Dimensões nas bordas
            this.pdf.setFontSize(6);
            this.pdf.setTextColor(255, 0, 0);

            // Largura na borda inferior
            const widthText = `${Math.round(piece.width)}`;
            this.pdf.text(widthText, x + (width / 2), y + height - 1, { align: 'center' });

            // Altura na borda direita (rotacionada 90 graus)
            const heightText = `${Math.round(piece.height)}`;
            // Mudamos para -90 graus para rotacionar no sentido correto
            const rad = (-90 * Math.PI) / 180;
            this.pdf.text(heightText, x + width - 2, y + (height / 2), {
             //   align: 'center',
                baseline: 'middle',
                transform: [
                    Math.cos(rad), Math.sin(rad),
                          -Math.sin(rad), Math.cos(rad),
                          x + width - 2, y + (height / 2)
                ],
                angle: 90 // Adicionamos esta propriedade para garantir a rotação
            });
        });

        this.pdf.setDrawColor(0);
        this.pdf.setLineWidth(0.2);
    }
}

// Atualiza o método generatePDF na classe CuttingPlan (fora da classe CuttingPlanPDFGenerator)
CuttingPlan.prototype.generatePDF = function() {
    if (typeof jspdf === 'undefined') {
        alert('Biblioteca jsPDF não carregada. Verifique se o script foi incluído.');
        return;
    }

    const pdfGenerator = new CuttingPlanPDFGenerator(this);
    pdfGenerator.generatePDF();
};
