require 'csv'

module Moveis3D
    class ComponentListDialog
        THICKNESS_TOLERANCE = 0.1
        TARGET_THICKNESS = 18.0
        SHEET_WIDTH = 2750.0  # mm
        SHEET_HEIGHT = 1850.0 # mm
        SAW_KERF = 4.0       # mm

        def initialize
            create_dialog_instance
            @dialog.set_size(1200, 800)
            @dialog.set_position(100, 100)
            @materials_components = {}  # Hash para agrupar componentes por material
        end

        def show
            generate_component_list
            @dialog.show
        end

        private

        # Função para carregar preços de um arquivo CSV
        def load_prices_from_csv(file_path)
            prices = {}
            CSV.foreach(file_path, headers: true) do |row|
                item_code = row['ItemCode']
                price = row['Preco'].to_f
                prices[item_code] = price
            end
            prices
        end

        def get_sheet_materials(definitions)
            materials = Set.new
            definitions.each do |definition|
                next unless definition.valid? && has_18mm_dimension?(definition)
                material = get_largest_faces_material(definition)
                materials.add(material) unless material == "Sem material"
            end
            materials.to_a
        end



        def update_price(item_code, new_price)
            begin
                csv_path = File.join(File.dirname(__FILE__), 'precos.csv')
                temp_path = File.join(File.dirname(__FILE__), 'precos_temp.csv')

                # Lê o arquivo CSV atual e cria um arquivo temporário
                CSV.open(temp_path, 'w') do |temp_csv|
                    temp_csv << ['ItemCode', 'Preco'] # headers

                    CSV.foreach(csv_path, headers: true) do |row|
                        if row['ItemCode'] == item_code
                            temp_csv << [item_code, new_price]
                        else
                            temp_csv << [row['ItemCode'], row['Preco']]
                        end
                    end
                end

                # Substitui o arquivo original pelo temporário
                File.delete(csv_path)
                File.rename(temp_path, csv_path)

                true
            rescue => e
                puts "Erro ao atualizar preço: #{e.message}"
                false
            end
        end

        def save_price(json_str)
            begin
                # Parse os dados JSON
                data = JSON.parse(json_str)
                csv_path = File.join(File.dirname(__FILE__), 'precos.csv')
                temp_path = File.join(File.dirname(__FILE__), 'precos_temp.csv')

                # Verifica se o arquivo existe, se não, cria com o cabeçalho
                unless File.exist?(csv_path)
                    File.write(csv_path, "ItemCode,Preco\n")
                end

                # Lê o arquivo existente para a memória
                rows = []
                found = false

                # Lê todas as linhas do arquivo CSV
                CSV.foreach(csv_path, headers: true) do |row|
                    if row['ItemCode'] == data['materialName']
                        # Atualiza o preço se encontrar o material
                        rows << [data['materialName'], data['price']]
                        found = true
                    else
                        # Mantém as outras linhas como estão
                        rows << [row['ItemCode'], row['Preco']]
                    end
                end

                # Se o material não foi encontrado, adiciona como novo
                unless found
                    rows << [data['materialName'], data['price']]
                end

                # Escreve todas as linhas no arquivo temporário
                CSV.open(temp_path, 'wb') do |csv|
                    # Escreve o cabeçalho
                    csv << ['ItemCode', 'Preco']
                    # Escreve todas as linhas
                    rows.each do |row|
                        csv << row
                    end
                end

                # Substitui o arquivo original pelo temporário
                File.delete(csv_path) if File.exist?(csv_path)
                File.rename(temp_path, csv_path)

                # Mostra mensagem de sucesso
                UI.messagebox("Preço atualizado com sucesso!")
                true
            rescue => e
                UI.messagebox("Erro ao atualizar preço: #{e.message}")
                puts "Erro detalhado: #{e.backtrace.join("\n")}"
                false
            end
        end


        def generate_sheet_materials_rows(materials_list, prices)
            rows = []
            materials_list.each do |material|
                # Busca o preço no arquivo CSV
                price = prices[material] || 0.0
                sheet_count = calculate_sheets_needed(material)

                rows << <<~HTML
                <tr data-itemcode="#{material}">
                <td>Chapas de MDF</td>
                <td>#{material}</td>
                <td><input type="number" class="quantity" value="#{sheet_count}" readonly></td>
                <td><input type="number" class="unit-cost" value="#{price}" step="0.01" min="0"></td>
                <td class="total-cost">R$ #{(sheet_count * price).round(2)}</td>
                </tr>
                HTML
            end
            rows.join("\n")
        end

        def calculate_sheets_needed(material)
            # Por padrão, retorna 1 até que o plano de corte seja calculado
            @sheets&.dig(material)&.length || 1
        end








        def create_dialog_instance
            @dialog = UI::HtmlDialog.new(
                dialog_title: "Plano de Corte - MDF 18mm",
                scrollable: true,
                width: 1200,
                height: 800,
                min_width: 400,
                min_height: 300,
                style: UI::HtmlDialog::STYLE_DIALOG
            )

            # Adicione este callback para salvar preços
            @dialog.add_action_callback("save_price") do |_context, json_str|
                save_price(json_str)
            end

            # Callback existente para fechar o diálogo
            @dialog.add_action_callback("closeDialog") do |_action_context|
                @dialog.close
            end
        end


        def mm_to_inch(mm)
            mm / 25.4
        end

        def has_18mm_dimension?(definition)
            return false unless definition.valid?
            bounds = definition.bounds
            return false if bounds.empty?
            target_thickness_inches = mm_to_inch(TARGET_THICKNESS)
            tolerance_inches = mm_to_inch(THICKNESS_TOLERANCE)
            dimensions = [bounds.depth, bounds.width, bounds.height]
            dimensions.count { |dim| (dim - target_thickness_inches).abs <= tolerance_inches } == 1
        end

        def get_face_materials(definition)
            return { main: "Sem material", edges: "Sem material" } unless definition.valid?

            faces = definition.entities.grep(Sketchup::Face)
            return { main: "Sem material", edges: "Sem material" } if faces.empty?

            # Converte as medidas para polegadas para comparação
            target_thickness_inches = mm_to_inch(TARGET_THICKNESS)
            tolerance_inches = mm_to_inch(THICKNESS_TOLERANCE)

            # Agrupa faces por suas normais
            faces_by_normal = {}
            edge_faces = []
            main_faces = []

            faces.each do |face|
                # Calcula as dimensões da face
                bounds = face.bounds
                dimensions = [bounds.width, bounds.height, bounds.depth]

                # Verifica se é uma face de borda (tem uma dimensão próxima a 18mm)
                is_edge = dimensions.any? { |dim| (dim - target_thickness_inches).abs <= tolerance_inches }

                normal = face.normal
                normal_key = normal.to_a.map { |n| n.round(6) }.join(',')

                if is_edge
                    edge_faces << face
                else
                    faces_by_normal[normal_key] ||= []
                    faces_by_normal[normal_key] << face
                end
            end

            # Encontra as faces principais (as maiores faces que não são bordas)
            main_material = "Sem material"
            edge_material = "Sem material"

            # Processa faces principais
            largest_area = 0
            faces_by_normal.each do |normal_key, normal_faces|
                opposite_normal = normal_faces.first.normal.reverse
                opposite_key = opposite_normal.to_a.map { |n| n.round(6) }.join(',')

                if faces_by_normal[opposite_key]
                    total_area = normal_faces.sum(&:area) + faces_by_normal[opposite_key].sum(&:area)
                    if total_area > largest_area
                        largest_area = total_area
                        main_faces = normal_faces + faces_by_normal[opposite_key]
                    end
                end
            end

            # Obtém o material das faces principais
            if !main_faces.empty?
                materials = main_faces.map { |face|
                                             [face.material, face.back_material]
                                           }.flatten.compact.uniq

                # Filtra materiais, ignorando "MDF Cru"
                valid_materials = materials.reject { |mat| mat.display_name == "MDF Cru" }

                main_material = if valid_materials.empty?
                "Sem material"
            else
                material = valid_materials.first
                material.display_name.empty? ? "Material padrão" : material.display_name
            end
        end

        # Obtém o material das bordas
        if !edge_faces.empty?
            edge_materials = edge_faces.map { |face|
                                              [face.material, face.back_material]
                                            }.flatten.compact.uniq

            # Filtra materiais das bordas, ignorando "MDF Cru"
            valid_edge_materials = edge_materials.reject { |mat| mat.display_name == "MDF Cru" }

            edge_material = if valid_edge_materials.empty?
            "Sem material"
        else
            material = valid_edge_materials.first
            material.display_name.empty? ? "Fita de Borda padrão" :
            "Fita de Borda #{material.display_name}"
        end
    end

    { main: main_material, edges: edge_material }
end

        def get_dimensions_mm(bounds)
            dimensions = [
                (bounds.width * 25.4).round(1),
                (bounds.height * 25.4).round(1),
                (bounds.depth * 25.4).round(1)
            ]

            non_18mm_dimensions = dimensions.reject do |dim|
                (dim - TARGET_THICKNESS).abs <= THICKNESS_TOLERANCE
            end

            non_18mm_dimensions.sort!

            {
                all: dimensions,
                cutting: non_18mm_dimensions
            }
        end

        def group_components_by_material(definitions)
            materials = {}
            definitions.each do |definition|
                next unless definition.valid? && has_18mm_dimension?(definition)

                # Verifica se existe pelo menos uma instância ativa do componente
                next if definition.instances.empty?

                material_name = get_largest_faces_material(definition)
                materials[material_name] ||= []
                materials[material_name] << definition
            end
            materials
        end

        def generate_material_options(materials)
            options = materials.keys.map do |material|
                "<option value='#{material}'>#{material}</option>"
            end
            options.join("\n")
        end

        # Função para contar componentes com ItemCode e buscar preços
        def count_components_with_itemcode(definitions, prices)
            components_with_itemcode = []
            definitions.each do |definition|
                # Verifica se a definição é válida e tem instâncias ativas
                next unless definition.valid? && !definition.instances.empty?

                definition.instances.each do |instance|
                    item_code = instance.get_attribute("ItemCode", "code")
                    if item_code
                        price = prices[item_code] || 0.0
                        existing_item = components_with_itemcode.find { |item| item[:code] == item_code }
                        if existing_item
                            existing_item[:count] += 1
                        else
                            components_with_itemcode << {
                                name: definition.name,
                                code: item_code,
                                count: 1,
                                price: price
                            }
                        end
                    end
                end
            end
            components_with_itemcode
        end





        def generate_component_list
            model = Sketchup.active_model
            definitions = model.definitions
            materials_hash = {}
            edge_materials = {}

            # Agrupa componentes por material principal e material de borda
            definitions.each do |definition|
                next unless definition.valid? && has_18mm_dimension?(definition)
                next if definition.instances.empty?

                materials = get_face_materials(definition)

                # Agrupa por material principal
                if materials[:main] != "Sem material"
                    materials_hash[materials[:main]] ||= []
                    materials_hash[materials[:main]] << definition
                end

                # Registra material das bordas
                if materials[:edges] != "Sem material"
                    edge_materials[materials[:edges]] ||= []
                    edge_materials[materials[:edges]] << definition
                end
            end

            # Carregar preços do CSV
            prices = load_prices_from_csv(File.join(File.dirname(__FILE__), 'precos.csv'))

            # Gerar linhas para todos os materiais
            all_material_rows = []

            # Adicionar linhas para chapas de MDF
            materials_hash.keys.each do |material|
                price = prices[material] || 0.0
                sheet_count = calculate_sheets_needed(material)

                all_material_rows << {
                    name: "Chapas de MDF",
                    code: material,
                    count: sheet_count,
                    price: price,
                    total: (sheet_count * price).round(2)
                }
            end

            # Adicionar linhas para fitas de borda
            edge_materials.each do |material, definitions|
                # Calcula o perímetro total das bordas
                total_edge_length = 0
                definitions.each do |definition|
                    bounds = definition.bounds
                    # Pega apenas as dimensões que não são 18mm
                    dims = get_dimensions_mm(bounds)[:cutting]
                    perimeter = dims.sum * 2 * definition.instances.length
                    total_edge_length += perimeter
                end

                # Converte para metros
                total_edge_length_m = (total_edge_length / 1000.0).round(2)

                price = prices[material] || 0.0
                all_material_rows << {
                    name: material,
                    code: material,
                    count: total_edge_length_m,
                    price: price,
                    total: (total_edge_length_m * price).round(2)
                }
            end

            # Adicionar linhas para componentes com ItemCode
            components_with_itemcode = count_components_with_itemcode(definitions, prices)
            all_material_rows.concat(components_with_itemcode)

            # Gerar HTML para todas as linhas
            material_rows_html = all_material_rows.map do |item|
                <<~HTML
                <tr data-itemcode="#{item[:code]}">
                <td>#{item[:name]}</td>
                <td>#{item[:code]}</td>
                <td><input type="number" class="quantity" value="#{item[:count]}" readonly></td>
                <td><input type="number" class="unit-cost" value="#{item[:price]}" step="0.01" min="0"></td>
                <td class="total-cost">R$ #{item[:total]}</td>
                </tr>
                HTML
            end.join("\n")

            # Carregar conteúdo dos arquivos externos
            css_path = File.join(File.dirname(__FILE__), 'css/styles.css')
            cssplan_path = File.join(File.dirname(__FILE__), 'css/cutting_plan.css')
            js_path = File.join(File.dirname(__FILE__), 'js/cutting_plan.js')
            jspdf_path = File.join(File.dirname(__FILE__), 'js/pdfGenerator.js')
            html_path = File.join(File.dirname(__FILE__), 'html/index.html')

            # Verificar e ler arquivos
            html_content = File.exist?(html_path) ? File.read(html_path) : ''
            css_content = File.exist?(css_path) ? File.read(css_path) : ''
            cssplan_content = File.exist?(cssplan_path) ? File.read(cssplan_path) : ''
            js_content = File.exist?(js_path) ? File.read(js_path) : ''
            jspdf_content = File.exist?(jspdf_path) ? File.read(jspdf_path) : ''

            # Substituir marcadores no HTML
            html_content.gsub!('{{STYLE_CONTENT}}', css_content)
            html_content.gsub!('{{STYLEPLAN_CONTENT}}', cssplan_content)
            html_content.gsub!('{{JAVASCRIPT_CONTENT}}', js_content)
            html_content.gsub!('{{JAVASCRIPTPDF_CONTENT}}', jspdf_content)
            html_content.gsub!('{{COMPONENTS_ROWS}}', generate_component_rows(materials_hash))
            html_content.gsub!('{{CURRENT_DATE}}', Time.now.strftime("%Y-%m-%d %H:%M:%S"))
            html_content.gsub!('{{MODEL_NAME}}', model.title.empty? ? "Sem título" : model.title)
            html_content.gsub!('{{MATERIAL_OPTIONS}}', generate_material_options(materials_hash))
            html_content.gsub!('{{SHEET_WIDTH}}', SHEET_WIDTH.to_s)
            html_content.gsub!('{{SHEET_HEIGHT}}', SHEET_HEIGHT.to_s)
            html_content.gsub!('<!--INSERT_ITEMCODE_ROWS-->', material_rows_html)

            @dialog.set_html(html_content)
        end


        def generate_component_rows(materials_hash)
            rows = []
            materials_hash.each do |material, definitions|
                rows << "<tr class='material-header' data-material='#{material}'><td colspan='5'>#{material}</td></tr>"
                definitions.each_with_index do |definition, index|
                    next unless definition.valid?
                    instances_count = definition.instances.length
                    dimensions = get_dimensions_mm(definition.bounds)
                    formatted_dimensions = dimensions[:cutting].join(" × ")
                    rows << "<tr data-dimensions='#{dimensions[:cutting].to_json}' data-material='#{material}'>"
                    rows << "<td><input type='checkbox' class='piece-selector' id='piece_#{material}_#{index}' checked='checked'></td>"
                    rows << "<td>#{definition.name}</td>"
                    rows << "<td>#{formatted_dimensions}</td>"
                    rows << "<td>#{instances_count}</td>"
                    rows << "<td><label class='rotation-toggle' title='Girar peça'><input type='checkbox' class='piece-rotation' data-piece-id='#{material}_#{index}'></label></td>"
                    rows << "</tr>"
                end
            end
            rows.join("\n")
        end
    end

    def self.show_component_list
        dialog = ComponentListDialog.new
        dialog.show
    end
end

unless file_loaded?(__FILE__)
    menu = UI.menu('Plugins')
    menu.add_item('Lista de Componentes - 18mm') { Moveis3D.show_component_list }
    file_loaded(__FILE__)
end
